/*
 * Copyright (C) 2018 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 * 
**/
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTableWidgetItem>
#include <QRadioButton>
#include <qgsettings.h>
#include <QFileSystemWatcher>
#include "customtype.h"
#include "contentpane.h"
#include "attributewindow.h"
#include "switchbutton.h"
#include "kswitchbutton.h"


#define KYLIN_USER_GUIDE_PATH "/"
#define KYLIN_USER_GUIDE_SERVICE "com.kylinUserGuide.hotel"
#define KYLIN_USER_GUIDE_INTERFACE "com.guide.hotel"

namespace Ui {
class MainWindow;
}
using namespace kdk;
class QLabel;
class AboutDialog;
class AttributeWindow;
class QProcess;
class LoadingDialog;

typedef std::shared_ptr<DeviceInfo> DeviceInfoPtr;
typedef QList<DeviceInfoPtr> DeviceList;
typedef QMap<int, DeviceList> DeviceMap;
class MainWindow : public QMainWindow
{
	Q_OBJECT

public:
	explicit MainWindow(QString usernameFromCmd, QWidget *parent = 0);
	~MainWindow();

protected:
    bool eventFilter(QObject *obj, QEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void resizeEvent(QResizeEvent *event);
    void paintEvent(QPaintEvent *);
/* Qt slots */
private slots:
    void on_btnDashBoard_clicked();
    void on_btnFingerPrint_clicked();
    void on_btnPermissions_clicked();
    void on_btnFingerVein_clicked();
    void on_btnIris_clicked();
    void on_btnVoicePrint_clicked();
    void on_btnFace_clicked();
    void on_btnStatus_clicked();
    void on_btnLogin_clicked();
    void on_btnSaver_clicked();
    void on_btnSystem_clicked();
    void on_lblNoDevicePFingerPrint_clicked();
    void on_lblNoDevicePFace_clicked();
    void on_lblNoDevicePFingerVein_clicked();
    void on_lblNoDevicePIris_clicked();
    void on_lblNoDevicePVoicePrint_clicked();
    void on_listWidgetDevicesType_currentIndexChanged(int);
    void on_tableWidgetDevices_cellDoubleClicked(int row, int column);
    void onDriverStatusClicked();
    void onDefaultDeviceChanged(bool checked);
    bool changeDeviceStatus(DeviceInfo *deviceInfo);
    void onUSBDeviceHotPlug(int, int, int);
    void onConfigurationChanged(QString key);
    void onUpdateDevice(int,int,int);
public slots:
    void onServiceStatusChanged(bool activate);
    void onReviceWindowMessage(QString message);
private:
	void prettify();
    void initSysMenu();
	void getDeviceInfo();
    void addContentPane(DeviceInfo *deviceInfo);
	void initialize();
    void initDeviceTypeList();
	void initBiometricPage();
    void updateBiometricPage(int type, int devNumNow);
	void initDashboardBioAuthSection();
    void initLoginSwitch();
    void initSaverSwitch();
    void initSystemSwitch();
    QPixmap *getUserAvatar(QString username);
    void setCurrentUser();
    void changeBtnColor(QPushButton *btn);
    void setVerificationStatus(bool status);
    void setLoginSwitch(bool status);
    void setSaverSwitch(bool status);
    void setSystemSwitch(bool status);
    int bioTypeToIndex(int type);
    bool restartService();
    void updateDevice();
    void updateDeviceListWidget(int biotype);
    void setDeviceStatus(QWidget *item, QLabel *label, QLabel *label1, bool connected, int flag);
    void raiseContentPane(DeviceInfo *deviceInfo);
    void setLastDeviceSelected();
    void sortContentPane();
    void showGuide(QString appName);
    int daemonIsNotRunning();
    void showAttributeWindow();
    void initPermissionWidget();
    QString getElidedText(QFont font,int width,QString strInfo);
    void setTText(QString text);
    void setStackedWidgetMain(QWidget* newWidget);

/* Members */
private:
	Ui::MainWindow *ui;
    QLabel  *lblStatus;
    QLabel *permissionLabel ;
    AttributeWindow *dialog = nullptr;
    //QPushButton *labelIcon;
    DeviceMap           deviceMap;
    //由于互斥的QButtonGroup会保证至少有一个被选中，所以这里不使用它
    QList<QRadioButton*> btnGroup;
	/* 用于和远端 DBus 对象交互的代理接口 */
    QDBusInterface *serviceInterface;
	int deviceCount;
    QMap<int, QList<DeviceInfo *>> deviceInfosMap;
	QMap<QString, ContentPane *> contentPaneMap;
	/* 通过命令行参数传入的用户名 */
    QString username;
    bool verificationStatus;    //生物识别开关状态
    bool loginSwitch;
    bool saverSwitch;
    bool systemSwitch;
    //for window move
    QPoint dragPos;
    bool dragWindow;

    QMenu *menu;
    AboutDialog *aboutDlg;

    /* 服务被关闭时提示 */
    QLabel *lblPrompt;
    DeviceInfo *lastDeviceInfo;
    QFileSystemWatcher *mWatcher;

    QColor m_colorBg;
    QGSettings *m_styleSettings = nullptr;
    float m_curFontSize;
    QString m_curStyle;

    QLabel *systemLabel;
    QLabel *saverLabel;
    QLabel *greetLabel;

    QProcess *m_procRestartService = nullptr;
    LoadingDialog *m_dialogLoading = nullptr;
};

#endif // MAINWINDOW_H
